#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("node:fs");
const path = require("node:path");
const vscode_markdown_languageservice_1 = require("@dsanders11/vscode-markdown-languageservice");
const minimist = require("minimist");
const node_fetch_1 = require("node-fetch");
const vscode_languageserver_1 = require("vscode-languageserver");
const vscode_uri_1 = require("vscode-uri");
const markdown_1 = require("../lib/markdown");
class NoOpLogger {
    constructor() {
        this.level = vscode_markdown_languageservice_1.LogLevel.Off;
    }
    log() { }
}
const diagnosticOptions = {
    ignoreLinks: [],
    validateDuplicateLinkDefinitions: vscode_markdown_languageservice_1.DiagnosticLevel.error,
    validateFileLinks: vscode_markdown_languageservice_1.DiagnosticLevel.error,
    validateFragmentLinks: vscode_markdown_languageservice_1.DiagnosticLevel.error,
    validateMarkdownFileLinkFragments: vscode_markdown_languageservice_1.DiagnosticLevel.error,
    validateReferences: vscode_markdown_languageservice_1.DiagnosticLevel.error,
    validateUnusedLinkDefinitions: vscode_markdown_languageservice_1.DiagnosticLevel.error,
};
async function fetchExternalLink(link, checkRedirects = false) {
    try {
        const response = await (0, node_fetch_1.default)(link);
        if (response.status !== 200) {
            console.log('Broken link', link, response.status, response.statusText);
        }
        else {
            if (checkRedirects && response.redirected) {
                const wwwUrl = new URL(link);
                wwwUrl.hostname = `www.${wwwUrl.hostname}`;
                // For now cut down on noise to find meaningful redirects
                const wwwRedirect = wwwUrl.toString() === response.url;
                const trailingSlashRedirect = `${link}/` === response.url;
                if (!wwwRedirect && !trailingSlashRedirect) {
                    console.log('Link redirection', link, '->', response.url);
                }
            }
            return true;
        }
    }
    catch (_a) {
        console.log('Broken link', link);
    }
    return false;
}
async function main(workspaceRoot, globs, { fetchExternalLinks = false, checkRedirects = false, ignoreGlobs = [] }) {
    var _a;
    const workspace = new markdown_1.DocsWorkspace(workspaceRoot, globs, ignoreGlobs);
    const parser = new markdown_1.MarkdownParser();
    const linkComputer = new markdown_1.MarkdownLinkComputer(workspace);
    const languageService = (0, vscode_markdown_languageservice_1.createLanguageService)({
        workspace,
        parser,
        logger: new NoOpLogger(),
        linkComputer,
    });
    const cts = new vscode_languageserver_1.CancellationTokenSource();
    let errors = false;
    const externalLinks = new Set();
    try {
        // Collect diagnostics for all documents in the workspace
        for (const document of await workspace.getAllMarkdownDocuments()) {
            for (let link of await languageService.getDocumentLinks(document, cts.token)) {
                if (link.target === undefined) {
                    link = (_a = (await languageService.resolveDocumentLink(link, cts.token))) !== null && _a !== void 0 ? _a : link;
                }
                if (link.target &&
                    link.target.startsWith('http') &&
                    new URL(link.target).hostname !== 'localhost') {
                    externalLinks.add(link.target);
                }
            }
            const diagnostics = await languageService.computeDiagnostics(document, diagnosticOptions, cts.token);
            if (diagnostics.length) {
                console.log('File Location:', path.relative(vscode_uri_1.URI.file(workspace.root).path, vscode_uri_1.URI.parse(document.uri).path));
            }
            for (const diagnostic of diagnostics) {
                console.log(`\tBroken link on line ${diagnostic.range.start.line + 1}:`, diagnostic.message);
                errors = true;
            }
        }
    }
    finally {
        cts.dispose();
    }
    if (fetchExternalLinks) {
        const externalLinkStates = await Promise.all(Array.from(externalLinks).map((link) => fetchExternalLink(link, checkRedirects)));
        errors = errors || !externalLinkStates.every((x) => x);
    }
    return errors;
}
function parseCommandLine() {
    const showUsage = (arg) => {
        if (!arg || arg.startsWith('-')) {
            console.log('Usage: electron-lint-markdown-links --root <dir> <globs> [-h|--help] [--fetch-external-links] ' +
                '[--check-redirects] [--ignore <globs>]');
            process.exit(1);
        }
        return true;
    };
    const opts = minimist(process.argv.slice(2), {
        boolean: ['help', 'fetch-external-links', 'check-redirects'],
        string: ['root', 'ignore', 'ignore-path'],
        unknown: showUsage,
    });
    if (opts.help || !opts.root || !opts._.length)
        showUsage();
    return opts;
}
if (require.main === module) {
    const opts = parseCommandLine();
    if (opts.ignore) {
        opts.ignore = Array.isArray(opts.ignore) ? opts.ignore : [opts.ignore];
    }
    else {
        opts.ignore = [];
    }
    if (opts['ignore-path']) {
        const ignores = fs.readFileSync(path.resolve(opts['ignore-path']), { encoding: 'utf-8' });
        for (const ignore of ignores.split('\n')) {
            opts.ignore.push(ignore.trimEnd());
        }
    }
    main(path.resolve(process.cwd(), opts.root), opts._, {
        fetchExternalLinks: opts['fetch-external-links'],
        checkRedirects: opts['check-redirects'],
        ignoreGlobs: opts.ignore,
    })
        .then((errors) => {
        if (errors)
            process.exit(1);
    })
        .catch((error) => {
        console.error(error);
        process.exit(1);
    });
}
//# sourceMappingURL=lint-markdown-links.js.map